@@ -3,7 +3,7 @@ package VCS::CMSynergy;
 # Copyright (c) 2001-2015 argumentum GmbH
 # See COPYRIGHT section in VCS/CMSynergy.pod for usage and distribution rights.
 
-our $VERSION = '1.45';
+our $VERSION = '1.46';
 
 use 5.008_001;                          # i.e. v5.8.1
 use strict;
@@ -26,7 +26,7 @@ use Type::Params qw( compile validate );
 use Types::Standard qw( slurpy Optional Str InstanceOf HasMethods Undef
     ArrayRef CodeRef GlobRef HashRef ScalarRef FileHandle Dict );
 use constant _PROJECT_SPEC   => Str | InstanceOf["VCS::CMSynergy::Project"];
-use constant _QUERY_KEYWORDS => compile(Str | ArrayRef | HashRef, _KEYWORDS);
+use constant _QUERY_KEYWORDS => compile((Str | ArrayRef | HashRef), _KEYWORDS);
 
 use constant ROW_HASH   => 1;
 use constant ROW_OBJECT => 2;
@@ -881,7 +881,7 @@ sub _history
     # (will be translated to undef by _query_result)
     foreach (qw( predecessors successors ))
     {
-        $want->{$_} = "%[$_]objectname" if $want->{$_};
+        $want->{$_} = "%{[$_]objectname[separator='\\n' null='']}" if $want->{$_};
     }
 
     my $format = $RS . join($FS, values %$want) . $FS;
@@ -902,8 +902,7 @@ sub _history
         {
             next unless $want->{$_};
             my $list = delete $row->{$_};       # temporarily strip slot
-            $cessors{$_} = defined $list ?
-                [ map { $self->object($_) } split(/,/, $list) ] : [];
+            $cessors{$_} = [ map { $self->object($_) } split(/\n/, $list) ];
         }
 
         if ($want->{object})
@@ -982,54 +981,52 @@ sub _history_classic
     }
     return \@result;
 }
-=for comment
 
-Consider the following history
+# Consider the following history
+# 
+#   ...  ->  foo,3  ->  foo,4  ->  foo,5  ->  bar,6  ->  bar,7  ->  ...
+# 
+# The "history" command reports only a partial view of that.
+# E.g. invoked as "ccm history foo,N:ascii:1" for some N it shows only the chain
+# of versions of "foo" plus "bar,6". predecessor/successor lists are 
+# artificially cut off at the "rename points",
+# so neither "ccm history foo,1:ascii:1" nor "ccm history bar,6:ascii:1" has the
+# complete predecessor/successor information. 
+# 
+# In order to produce a complete history it is _not_ enough to simply
+# "glue" together the graphs for foo,N and bar,N. We must form the
+# "union" of predecessor/successor lists for the overlapping elements.
+# 
+#   ccm history foo,5:ascii:1
+# 
+#   foo,5:ascii:1
+#   Predecessors:
+#           foo,4:ascii:1
+#   Successors:
+#           bar,6:ascii:1
+#   ***************************************************************
+#   bar,6:ascii:1
+#   Predecessors:
+#           foo,5:ascii:1
+#   Successors:
+#   ***************************************************************
+# 
+#   ccm history bar,6:ascii:1
+# 
+#   foo,5:ascii:1
+#   Predecessors:
+#   Successors:
+#           bar,6:ascii:1
+#   ***************************************************************
+#   bar,6:ascii:1
+#   Predecessors:
+#           foo,5:ascii:1
+#   Successors:
+#           bar,7:ascii:1
+#   ***************************************************************
 
-  ...  ->  foo,3  ->  foo,4  ->  foo,5  ->  bar,6  ->  bar,7  ->  ...
 
-The "history" command reports only a partial view of that.
-E.g. invoked as "ccm history foo,N:ascii:1" for some N it shows only the chain
-of versions of "foo" plus "bar,6". predecessor/successor lists are 
-artificially cut off at the "rename points",
-so neither "ccm history foo,1:ascii:1" nor "ccm history bar,6:ascii:1" has the
-complete predecessor/successor information. 
-
-In order to produce a complete history it is _not_ enough to simply
-"glue" together the graphs for foo,N and bar,N. We must form the
-"union" of predecessor/successor lists for the overlapping elements.
-
-  ccm history foo,5:ascii:1
-
-  foo,5:ascii:1
-  Predecessors:
-          foo,4:ascii:1
-  Successors:
-          bar,6:ascii:1
-  ***************************************************************
-  bar,6:ascii:1
-  Predecessors:
-          foo,5:ascii:1
-  Successors:
-  ***************************************************************
-
-  ccm history bar,6:ascii:1
-
-  foo,5:ascii:1
-  Predecessors:
-  Successors:
-          bar,6:ascii:1
-  ***************************************************************
-  bar,6:ascii:1
-  Predecessors:
-          foo,5:ascii:1
-  Successors:
-          bar,7:ascii:1
-  ***************************************************************
-
-=cut
-
-# FIXME pod
+# FIXME needs pod
 sub full_history_arrayref
 {
     my $self = shift;
@@ -1038,7 +1035,7 @@ sub full_history_arrayref
     return _flatten_rows($self->_full_history($file_spec, $keywords), $keywords);
 }
 
-# FIXME pod
+# FIXME needs pod
 sub full_history_hashref
 {
     my $self = shift;
@@ -1350,7 +1347,7 @@ sub project_tree
 {
     my $self = shift;
     my ($options, $projects) = 
-        validate(\@_, Undef | HashRef, slurpy ArrayRef[_PROJECT_SPEC]);
+        validate(\@_, (Undef | HashRef), slurpy ArrayRef[_PROJECT_SPEC]);
 
     # make a copy of $options, because we're modifying it below
     my %wanted = %{ $options || {} };           # Note: $options may be undef
@@ -1394,7 +1391,7 @@ sub project_diff
 {
     my $self = shift;
     my ($options, $old_project, $new_project, $differ) = 
-        validate(\@_, Undef | HashRef, _PROJECT_SPEC, _PROJECT_SPEC, HasMethods[qw( added deleted changed )]);
+        validate(\@_, (Undef | HashRef), _PROJECT_SPEC, _PROJECT_SPEC, HasMethods[qw( added deleted changed )]);
 
     # make a copy of $options, because we're modifying it below
     my %opts = %{ $options || {} };     # Note: $options may be undef
@@ -1573,7 +1570,7 @@ sub copy_attribute
     my @flags;
     @flags = map { "-$_" } @{ splice(@_, 1, 1) } if defined $_[1] && ref $_[1] eq "ARRAY";
     
-    my ($names, $file_specs) = validate(\@_, Str | ArrayRef[Str], slurpy ArrayRef[_FILE_SPEC]);
+    my ($names, $file_specs) = validate(\@_, (Str | ArrayRef[Str]), slurpy ArrayRef[_FILE_SPEC]);
     $names = join(':', @$names) if ref $names eq "ARRAY";
 
     return scalar $self->ccm(qw/attribute -copy/, $names, @flags, @$file_specs);
@@ -1640,7 +1637,7 @@ sub property
 {
     my $self = shift;
     my ($keyword_s, $file_spec) = 
-        validate(\@_, ( Str | ArrayRef[Str] ), _FILE_SPEC);
+        validate(\@_, (Str | ArrayRef[Str]), _FILE_SPEC);
 
     if (ref $keyword_s)
     {
@@ -2108,7 +2105,7 @@ sub object_from_proj_ref
 {
     my $self = shift;
     my ($path, $proj_spec, $keywords) =
-        validate(\@_, Str | ArrayRef[Str], _PROJECT_SPEC, _KEYWORDS);
+        validate(\@_, (Str | ArrayRef[Str]), _PROJECT_SPEC, _KEYWORDS);
 
     $path = join(VCS::CMSynergy::Client::_pathsep, @$path) if ref $path; 
     $proj_spec = $proj_spec->displayname if ref $proj_spec;
