@@ -302,7 +302,7 @@ sub traverse
 {
     my $self = shift;
     my ($arg_wanted, $dir) = 
-        validate(\@_, CodeRef | HashRef, Optional[InstanceOf["VCS::CMSynergy::Object"]]);
+        validate(\@_, (CodeRef | HashRef), Optional[InstanceOf["VCS::CMSynergy::Object"]]);
 
     my %wanted;
     if (ref $arg_wanted eq 'CODE')
@@ -654,22 +654,25 @@ sub has_child
 }
 
 
-=head2 object_from_proj_ref
+=head2 object_from_path
 
-  $obj = $proj->object_from_proj_ref($path, @keywords);
-  $obj = $proj->object_from_proj_ref(\@path_components, @keywords);
+  $obj = $proj->object_from_path($path, @keywords);
+  $obj = $proj->object_from_path(\@path_components, @keywords);
 
-is exactly the same as 
+Returns the C<VCS::CMSynergy::Object> identified by
+workarea path C<$path> in project C<$proj>.
 
   $obj = $proj->ccm->object_from_proj_ref($path, $proj, @keywords);
   $obj = $proj->ccm->object_from_proj_ref(\@path_components, $proj, @keywords);
 
+See L<VCS::CMSynergy/object_from_proj_ref> for details.
+
 =cut
 
-sub object_from_proj_ref
+sub object_from_path
 {
     my $self = shift;
-    my ($path, $keywords) = validate(\@_, Str | ArrayRef[Str], _KEYWORDS);
+    my ($path, $keywords) = validate(\@_, (Str | ArrayRef[Str]), _KEYWORDS);
 
     return $self->ccm->object_from_proj_ref($path, $self, @$keywords);
 }
